#!/usr/bin/env python3
"""CI tasks."""
from codecs import ignore_errors
from functools import partial
import shutil
import glob
import os
from doit.tools import create_folder
from doit.task import clean_targets

DOIT_CONFIG = {'default_tasks': ['translations']}


def task_pot():
    """Re-create .pot ."""
    return {
        "actions": ['pybabel extract -F locales/babel-mapping.ini -o jackalify.pot jackalify'],
        "file_dep": glob.glob('**/*.py', recursive=True),
        "targets": ['jackalify.pot'],
        "clean": True,
    }


def task_po():
    """Update translations."""
    return {
        "actions": ['pybabel update -D jackalify -d locales -i jackalify.pot'],
        "file_dep": ['jackalify.pot'],
        "task_dep": ['pot'],
        "targets": glob.glob("locales/**/*.po", recursive=True),
    }


def task_translations():
    """Compile translations."""
    languages = ['ru', 'en']
    actions = []
    for lang in languages:
        actions.append((create_folder, [f'jackalify/locales/{lang}/LC_MESSAGES']))
        actions.append(f'pybabel compile -i locales/{lang}/LC_MESSAGES/jackalify.po -o jackalify/locales/{lang}/LC_MESSAGES/jackalify.mo -l {lang}')
    return {
        "actions": actions,
        "file_dep": glob.glob("locales/**/*.po", recursive=True),
        "task_dep": ['po'],
        "targets": glob.glob("jackalify/locales/**/*.mo", recursive=True) if glob.glob("jackalify/locales/**/*.mo", recursive=True) else ['.mo'],
        "clean": True,
    }


def task_translations_ru():
    """Compile translations ru."""
    languages = ['ru']
    actions = []
    for lang in languages:
        actions.append((create_folder, [f'jackalify/locales/{lang}/LC_MESSAGES']))
        actions.append(f'pybabel compile -i locales/{lang}/LC_MESSAGES/jackalify.po -o jackalify/locales/{lang}/LC_MESSAGES/jackalify.mo -l {lang}')
    return {
        "actions": actions,
    }


def task_html():
    """Make HTML documentation."""
    build_dir = 'docs/_build'
    clean_build = partial(shutil.rmtree, build_dir, ignore_errors=True)
    return {
        "actions": ['sphinx-build docs %(targets)s'],
        "file_dep": glob.glob("**/*.py", recursive=True) + glob.glob("**/*.rst", recursive=True),
        "task_dep": ["translations"],
        "targets": [build_dir],
        "clean": [clean_build],
    }


def task_test():
    """Perform all tests."""
    return {
        "actions": [],
        "task_dep": ["unittest", "style", "docstyle"],
    }


def task_unittest():
    """Perform unittests."""
    return {
        "actions": ['python -m unittest -v -f test/*.py'],
        "task_dep": ["translations"],
    }


def task_style():
    """Check style against flake8."""
    return {
        "actions": ['flake8 --max-line-length=120 jackalify'],
    }


def task_docstyle():
    """Check docstrings against pydocstyle."""
    return {
        "actions": ['pydocstyle jackalify'],
    }


def rm_dir(dir):
    """Remove dir if empty, do nothing if not"""
    try:
        os.rmdir(dir)
    except:
        pass


def task_wheel():
    """Create binary wheel distribution."""
    clean_dist = partial(rm_dir, 'dist')
    clean_build = partial(shutil.rmtree, 'build', ignore_errors=True)
    clean_egg = partial(shutil.rmtree, 'jackalify.egg-info', ignore_errors=True)
    return {
        "actions": ['python -m build -w'],
        "verbosity": 2,
        "task_dep": ['translations'],
        "targets": glob.glob("dist/*.whl") if glob.glob("dist/*.whl") else ['.whl'],
        "clean": [clean_targets, clean_dist, clean_build, clean_egg],
    }


def task_source():
    """Create source distribution."""
    clean_dist = partial(rm_dir, 'dist')
    clean_egg = partial(shutil.rmtree, 'jackalify.egg-info', ignore_errors=True)
    return {
        "actions": ['python -m build -s'],
        "verbosity": 2,
        "targets": glob.glob("dist/*.tar.gz") if glob.glob("dist/*.tar.gz") else ['.tar.gz'],
        "clean": [clean_targets, clean_dist, clean_egg],
    }


def task_release():
    """Release wheel and source distribution to test.pypi"""
    return {
        "actions": ["python -m twine upload --repository pypi --config-file .pypirc dist/*"],
        "task_dep": ['wheel', 'source']
    }



import os
import subprocess
import unittest


class TestCli(unittest.TestCase):

    def setUp(self) -> None:
        self.data_dir = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
        self.img_name = "black"
        self.root_path = os.path.abspath(os.path.join(os.path.dirname(__file__), ".."))
        self.img_path = os.path.join(self.data_dir, f"{self.img_name}.png")

    def test_make_img(self):
        res = subprocess.run(["python", "-m", 'jackalify', self.img_path])
        self.assertEqual(0, res.returncode)
        self.assertTrue(os.path.isfile(os.path.join(self.data_dir, f"{self.img_name}_jackalified.png")))
        os.remove(os.path.join(self.data_dir, f"{self.img_name}_jackalified.png"))

        res = subprocess.run(["python", "-m", 'jackalify', "-o", self.img_path, self.img_path])
        self.assertEqual(0, res.returncode)
        self.assertTrue(os.path.isfile(os.path.join(self.data_dir, f"{self.img_name}_jackalified.png")))
        os.remove(os.path.join(self.data_dir, f"{self.img_name}_jackalified.png"))

        res = subprocess.run(["python", "-m", 'jackalify', "-o", "other.png", self.img_path])
        self.assertEqual(0, res.returncode)
        self.assertTrue(os.path.isfile("other.png"))
        os.remove("other.png")

    def test_make_gif(self):
        res = subprocess.run(["python", "-m", 'jackalify', "-g", self.img_path])
        self.assertEqual(0, res.returncode)
        self.assertTrue(os.path.isfile(os.path.join(self.data_dir, f"{self.img_name}_jackalified.gif")))
        os.remove(os.path.join(self.data_dir, f"{self.img_name}_jackalified.gif"))

        res = subprocess.run(["python", "-m", 'jackalify', "-g", "-o", "other.gif", self.img_path])
        self.assertEqual(0, res.returncode)
        self.assertTrue(os.path.isfile("other.gif"))
        os.remove("other.gif")

    def test_non_existent(self):
        non_existent_path = os.path.join(self.data_dir, "non_existing.png")
        self.assertFalse(os.path.isfile(non_existent_path))
        res = subprocess.run(["python", "-m", 'jackalify', non_existent_path])
        self.assertEqual(2, res.returncode)

    def test_wrong_file(self):
        res = subprocess.run(["python", "-m", 'jackalify', self.data_dir])
        self.assertEqual(2, res.returncode)

    def test_wrong_args(self):
        res = subprocess.run(["python", "-m", 'jackalify'])
        self.assertEqual(2, res.returncode)

        res = subprocess.run(["python", "-m", 'jackalify', "-other", self.img_path])
        self.assertEqual(2, res.returncode)

        res = subprocess.run(["python", "-m", 'jackalify', self.img_path, "other.png"])
        self.assertEqual(2, res.returncode)

        res = subprocess.run(["python", "-m", 'jackalify', self.img_path, "-o", self.img_path, "-g"])
        self.assertEqual(2, res.returncode)

        res = subprocess.run(["python", "-m", 'jackalify', "-g", self.img_path, "-o", "other.mkv"])
        self.assertEqual(2, res.returncode)

        res = subprocess.run(["python", "-m", 'jackalify', self.img_path, "-w"])
        self.assertEqual(2, res.returncode)



"""Fastapi server interface."""
import os
from typing import Dict

import aiofiles
from fastapi import FastAPI, File, UploadFile, HTTPException, BackgroundTasks
from fastapi.responses import HTMLResponse
from fastapi.staticfiles import StaticFiles
from starlette.requests import Request
from starlette.templating import Jinja2Templates, _TemplateResponse
import gettext

from jackalify.jackal import jackalify, getProgress  # noqa


app = FastAPI()
fastapi_path = os.path.abspath(os.path.dirname(__file__))
translation = gettext.translation('jackalify', localedir=os.path.join(fastapi_path, "locales"), languages=['en', 'ru'])
templates = Jinja2Templates(directory=os.path.join(fastapi_path, 'templates'))
templates.env.add_extension('jinja2.ext.i18n')
templates.env.install_gettext_translations(translation)
app.mount('/static', StaticFiles(directory=os.path.join(fastapi_path, 'static')), name='static')
working_dir = os.path.join(fastapi_path, 'static', 'working')
os.makedirs(working_dir, exist_ok=True)

picture = None
video = None


@app.post("/", response_class=HTMLResponse)
async def create_jacklified(
        request: Request, background_tasks: BackgroundTasks,
        file: UploadFile = File(...)
) -> _TemplateResponse:
    """Upload file and call 'jackalify algorithm.

    :param request: Request object.
    :type request: Request
    :param background_tasks: Class for execution task on the background.
    :type background_tasks: BackgroundTasks
    :param file: Object for file from form.
    :type file: UploadFiles
    :return: Response object.
    :rtype: _TemplateResponse
    """
    global picture
    global video

    for file_name in os.listdir(working_dir):
        file_path = os.path.join(working_dir, file_name)
        os.remove(file_path)

    if not file.filename.lower().endswith(('.png', '.jpg', '.jpeg')):
        raise HTTPException(status_code=415, detail="Wrong file extension: only  png, jpg, jpeg are allowed")

    file_name, file_extension = os.path.splitext(file.filename)
    async with aiofiles.open(os.path.join(working_dir, f'source{file_extension}'), 'wb') as out_file:
        content = await file.read()
        picture = f'source{file_extension}'
        await out_file.write(content)

    background_tasks.add_task(jackalify,
                              os.path.join(working_dir, f'source{file_extension}'),
                              os.path.join(working_dir, 'jackalified.gif'),
                              os.path.join(working_dir, 'jackalified.png'))

    video = 'jackalified.gif'

    return templates.TemplateResponse('main_form.html',
                                      {'request': request,
                                       'picture': picture,
                                       'video': video,
                                       'photo': 'jackalified.png'})


@app.get("/", response_class=HTMLResponse)
async def show_jackalified(request: Request) -> _TemplateResponse:
    """Show souce and jackalified files.

    :param request: Request object.
    :type request: Request
    :return: Response object.
    :rtype: _TemplateResponse
    """
    global picture
    global video

    return templates.TemplateResponse('main_form.html',
                                      {'request': request,
                                       'picture': picture,
                                       'video': video,
                                       'photo': 'jackalified.png'})


@app.get("/checkGIF")
async def checkGIF_fastapi() -> Dict:
    """Check if GIF is created.

    :return: Indicator of GIF creation and path to GIF.
    :rtype: Dict
    """
    gif_path = os.path.join(working_dir, 'jackalified.gif')
    is_exists = os.path.exists(gif_path)
    return {'answ': is_exists, 'path': 'jackalified.gif'}


@app.get("/getProgress")
async def getProgress_fastapi() -> Dict:
    """Get the progress of GIF creating.

    :return: Progress of GIF creating.
    :rtype: Dict
    """
    return {'prg': getProgress()}


if __name__ == '__main__':
    import uvicorn
    os.makedirs(working_dir, exist_ok=True)
    uvicorn.run(app)
    for file_name in os.listdir(working_dir):
        file_path = os.path.join(working_dir, file_name)
        os.remove(file_path)



"""Jackalify module."""

import argparse
import os
import subprocess
import sys
import gettext

from jackalify.jackal import jackalify

project_dir = os.path.dirname(__file__)
translation = gettext.translation('jackalify', localedir=os.path.join(project_dir, 'locales'), languages=['en', 'ru'])
_ = translation.gettext


def is_valid_file(parser: argparse.ArgumentParser, arg: str) -> str:
    """Check if file exists.

    :param parser: Argparse parser object.
    :type parser: argparse.ArgumentParser
    :param arg: Argument containing path po file.
    :type arg: str
    :return: Path to file if it exists.
    :rtype: str
    """
    path, extention = os.path.splitext(arg)
    if not os.path.exists(arg):
        parser.error(str.format(_("The file {} does not exist!"), arg))
    elif extention.lower() not in ['.png', '.jpg', '.jpeg']:
        parser.error(str.format(_("Wrong file extension '{}'! Try '.png', '.jpg', or '.jpeg' file!"), extention))
    else:
        return arg


def main():
    """CLI functionality."""
    parser = argparse.ArgumentParser(description=_('Jackalifying algorithm'))
    parser.add_argument(
        '-w', '--web',
        action='store_true',
        help=_('run fastapi server interface')
    )
    parser.add_argument(
        '-g', '--gif',
        action='store_true',
        help=_("create jackalified gif instead of picture")
    )
    parser.add_argument(
        'input_path',
        nargs='?',
        action='store',
        help=_('picture you want to jackalify'),
        type=lambda x: is_valid_file(parser, x)
    )
    parser.add_argument(
        '-o', '--output',
        action='store',
        dest='output_path',
        help=_('path to save jackalified instance'),
    )
    args = parser.parse_args()
    if args.web:
        if len(sys.argv) > 2:
            parser.error(_("-w must be a single argument!"))
        else:
            script_path = os.path.abspath(os.path.join(os.path.dirname(__file__), "app.py"))
            subprocess.run(["python", script_path])
    elif args.input_path:
        if args.output_path:
            if args.input_path == args.output_path:
                if args.gif:
                    parser.error(_("Output file shouldn't be gif!"))
                path, extention = os.path.splitext(args.output_path)
                args.output_path = f"{path}_jackalified{extention}"
            if args.gif:
                if not args.output_path.lower().endswith(".gif"):
                    parser.error(_("Output name should end with '.gif'!"))
                jackalify(args.input_path, video_path=args.output_path)
            else:
                if not args.output_path.lower().endswith(('.png', '.jpg', '.jpeg')):
                    parser.error(_("Output name should end with '.png', '.jpg' or '.jpeg'!"))
                jackalify(args.input_path, out_image_path=args.output_path)
        else:
            path, extention = os.path.splitext(args.input_path)
            if args.gif:
                output_path = f"{path}_jackalified.gif"
                jackalify(args.input_path, video_path=output_path)
            else:
                output_path = f"{path}_jackalified{extention}"
                jackalify(args.input_path, out_image_path=output_path)
    else:
        parser.error(_("No input path given!"))



"""A wrapper for getting a video with the distortion process from an image."""
import gettext
import io
import os
import sys
from typing import Optional

import numpy as np
from PIL import Image
import cv2
from tqdm import tqdm

from jackalify.seam_carve import seam_carve  # noqa

project_dir = os.path.dirname(__file__)
translation = gettext.translation('jackalify', localedir=os.path.join(project_dir, 'locales'), languages=['en', 'ru'])
_ = translation.gettext

it = 0
max_it = 0


def jackalify(in_image_path: str, video_path: Optional[str] = None, out_image_path: Optional[str] = None):
    """Apply the seam carving algorithm to the image and get a video with the distortion process.

    :param in_image_path: The path to the input image.
    :type in_image_path: str
    :param video_path: The path to the output video (if None then no video would be generated).
    :type video_path: str
    :param out_image_path: The path to the output image (if None then no photo would be generated).
    :type out_image_path: str
    """
    global it
    global max_it

    if not video_path and not out_image_path:
        raise AttributeError("At least one of output paths should be defined!")

    image = cv2.cv2.cvtColor(cv2.cv2.imread(in_image_path), cv2.cv2.COLOR_BGR2RGB)
    image = cv2.cv2.resize(
        image,
        (
            image.shape[1] * 512 // max(image.shape[:2]),
            image.shape[0] * 512 // max(image.shape[:2]),
        ),
    )

    height, width, _ = image.shape

    frames = []

    max_it = int(min(height, width) * 0.75)
    for it in tqdm(range(max_it)):
        image = seam_carve(image, 'horizontal')
        image = seam_carve(image, 'vertical')
        frames.append(Image.fromarray(np.uint8(cv2.resize(image, (width, height))), mode="RGB"))

    if out_image_path:
        frames[-1].save(out_image_path)

    if video_path:
        animated_gif = io.BytesIO()
        frames[0].save(
            animated_gif,
            format="GIF",
            save_all=True,
            append_images=frames[1:],
            optimize=True,
            duration=25,
            loop=0
        )
        animated_gif.seek(0)
        with open(video_path, 'wb') as gif_file:
            gif_file.write(animated_gif.read())


def getProgress():
    """Get progress of jackalify."""
    global it
    global max_it
    return (it * 100 // max_it) if max_it > 0 else 0


def main():
    """Run the main function."""
    message = _('Usage – python jackalify.py input_image_path output_video_path')
    if len(sys.argv) != 3:
        raise RuntimeError(message)

    jackalify(sys.argv[1], sys.argv[2])
